﻿using GridAppSample.Common;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using Windows.Storage;
using Windows.Storage.Streams;
using WinRTDatabase;

//완전 중요!!!
using System.Runtime.InteropServices.WindowsRuntime;

namespace GridAppSample
{
    public class SamplePageViewModel : GridAppSample.Common.BindableBase
    {
        public const string DatabaseName = "SampleDb";

        /// <summary>
        /// 데이터베이스
        /// </summary>
        private Database _database;

        private Table<PersonModel> people;
        /// <summary>
        /// 피플 테이블
        /// </summary>
        public Table<PersonModel> People
        {
            get { return people; }
            set
            {
                people = value; 
                OnPropertyChanged();
            }
        }

        private PersonModel selectedPerson;
        /// <summary>
        /// 선택된 사람
        /// </summary>
        public PersonModel SelectedPerson
        {
            get { return selectedPerson; }
            set 
            { 
                selectedPerson = value;
                OnPropertyChanged();
            }
        }


        public SamplePageViewModel()
        {
            if (Windows.ApplicationModel.DesignMode.DesignModeEnabled == false)
            {
                Initialise();
            }
            else
            {
                People = new Table<PersonModel>();
                People.Add(new PersonModel { FirstName = "Park", LastName = "kaki104", Age = 11 });
                People.Add(new PersonModel { FirstName = "Park", LastName = "2222222", Age = 22 });
                People.Add(new PersonModel { FirstName = "Park", LastName = "kaki103", Age = 33 });
                People.Add(new PersonModel { FirstName = "Park", LastName = "4444444", Age = 44 });
                People.Add(new PersonModel { FirstName = "Park", LastName = "kaki105", Age = 55 });
            }
        }

        public async void Initialise()
        {
            var exists = await Database.DoesDatabaseExistsAsync(DatabaseName, StorageLocation.Local);
            //db존재여부 확인
            if (!exists)
            {
                _database = await Database.CreateDatabaseAsync(DatabaseName, StorageLocation.Local);
                //파일 생성시 이름을 주는 방법도 있는데 사용하면 오류남..
                _database.CreateTable<PersonModel>();
                //변경사항 저장
                await _database.SaveAsync();
            }
            else
            {
                //db열기
                _database = await Database.OpenDatabaseAsync(DatabaseName, true, StorageLocation.Local);
            }
            People = await _database.Table<PersonModel>();

            _database.PropertyChanged += new PropertyChangedEventHandler(_database_PropertyChanged);
        }

        void _database_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
        }

        private DelegateCommand addCommand;
        /// <summary>
        /// 추가
        /// </summary>
        public DelegateCommand AddCommand
        {
            get
            {
                if (addCommand == null)
                {
                    addCommand = new DelegateCommand(
                        async _ =>
                        {
                            var newItem = new PersonModel 
                            {
                                PersonID = new Guid(),
                                FirstName = "Input First Name",
                                LastName = "Input Last Name",
                                Age = 0
                            };
                            People.Add(newItem);
                            await _database.SaveAsync();
                        });
                }
                return addCommand;
            }
        }

        private DelegateCommand removeCommand;
        /// <summary>
        /// 삭제
        /// </summary>
        public DelegateCommand RemoveCommand
        {
            get
            {
                if (removeCommand == null)
                {
                    removeCommand = new DelegateCommand(
                        async _ =>
                        {
                            if (SelectedPerson != null)
                            {
                                People.Remove(SelectedPerson);
                                await _database.SaveAsync();
                            }
                        });
                }
                return removeCommand;
            }
        }

        private DelegateCommand updateCommand;
        /// <summary>
        /// 수정
        /// </summary>
        public DelegateCommand UpdateCommand
        {
            get
            {
                if (updateCommand == null)
                {
                    updateCommand = new DelegateCommand(
                        async _ =>
                        {
                            await _database.SaveAsync();
                        });
                }
                return updateCommand;
            }
        }

    }
}
